#include "ADTFifo.h"
#include <string.h>

int ADTFifo_Init (tADTFifo* pFifo, void* pBufBegin, size_t SizeOfItem,
                  size_t ItemsPerBuffer)
{
  memset (pFifo, 0, sizeof(tADTFifo));
  if
  (
    (pFifo == NULL) ||
    (pBufBegin == NULL) ||
    (!SizeOfItem) ||
    (!ItemsPerBuffer)
  )
    return 0;
  
  pFifo->pBufRead = 
    pFifo->pBufWrite = 
      pFifo->pBufBegin = pBufBegin;
  pFifo->SizeOfItem    = SizeOfItem;
  pFifo->ItemsPerBuffer = ItemsPerBuffer;
  pFifo->SizeOfBuf     = SizeOfItem*ItemsPerBuffer;
  pFifo->pBufEnd       = (char*)pBufBegin + pFifo->SizeOfBuf;

  pFifo->PowerOf2DivShift = -1;
  if (!(SizeOfItem & (SizeOfItem-1)))
  {
    while (SizeOfItem)
    {
      pFifo->PowerOf2DivShift++;
      SizeOfItem >>= 1;
    }
  }

  return 1;
}

size_t ADTFifo_ReadAvail (tADTFifo* pFifo)
{
  ptrdiff_t diff = (char*)pFifo->pBufWrite - (char*)pFifo->pBufRead;
  if (diff < 0)
    diff += pFifo->SizeOfBuf;
  return (pFifo->PowerOf2DivShift >= 0)?
    diff >> pFifo->PowerOf2DivShift:
    diff / pFifo->SizeOfItem;
}

size_t ADTFifo_WriteAvail (tADTFifo* pFifo)
{
  size_t res = ADTFifo_ReadAvail (pFifo);
  if (pFifo->ItemsPerBuffer-1 > res)
    return pFifo->ItemsPerBuffer-1 - res;
  else
    return 0;
}

void ADTFifo_Read (tADTFifo* pFifo, void* pItem)
{
  memcpy (pItem, pFifo->pBufRead, pFifo->SizeOfItem);
  {
    char* p = pFifo->pBufRead;
    pFifo->pBufRead = p + pFifo->SizeOfItem;
  }
  if ((char*)pFifo->pBufRead >= (char*)pFifo->pBufEnd)
    pFifo->pBufRead = pFifo->pBufBegin;
}

void ADTFifo_Write (tADTFifo* pFifo, const void* pItem)
{
  memcpy (pFifo->pBufWrite, pItem, pFifo->SizeOfItem);
  {
    char* p = pFifo->pBufWrite;
    pFifo->pBufWrite = p + pFifo->SizeOfItem;
  }
  if ((char*)pFifo->pBufWrite >= (char*)pFifo->pBufEnd)
    pFifo->pBufWrite = pFifo->pBufBegin;
}

#if 0
#include <stdio.h>
#define ITEMS_PER_BUFFER 11
int main()
{
  tADTFifo Fifo;
  int aFifoBuffer[ITEMS_PER_BUFFER];
  int i;
  ADTFifo_Init (&Fifo, aFifoBuffer, sizeof(aFifoBuffer[0]), ITEMS_PER_BUFFER);
  printf ("ReadAvail: %d, WriteAvail: %d\n", 
          (int)ADTFifo_ReadAvail(&Fifo), (int)ADTFifo_WriteAvail(&Fifo));
  for (i=0;i<ITEMS_PER_BUFFER-1;i++)
  {
    printf ("item: %d\n", i);
    ADTFifo_Write (&Fifo, &i);
  }
  printf ("ReadAvail: %d, WriteAvail: %d\n",
          (int)ADTFifo_ReadAvail(&Fifo), (int)ADTFifo_WriteAvail(&Fifo));
  for (i=0;i<ITEMS_PER_BUFFER-1;i++)
  {
    int tmp;
    ADTFifo_Read (&Fifo, &tmp);
    printf ("item: %d\n", tmp);
  }
  printf ("ReadAvail: %d, WriteAvail: %d\n",
          (int)ADTFifo_ReadAvail(&Fifo), (int)ADTFifo_WriteAvail(&Fifo));
  return 0;
}
#endif
