#ifndef _DISPLAY_H_
#define _DISPLAY_H_

#define DOS_DEBUG 01

#include "../ComTypes.h"
#include "../ComDefs.h"

#define COLOR_MAKE(a,r,g,b)\
(\
  (((uint32)((a)&0xff))<<24)|\
  (((uint32)((r)&0xff))<<16)|\
  (((uint32)((g)&0xff))<< 8)|\
  ( (uint32)((b)&0xff)     )\
)

#define COLOR_GET_ALPHA(c)      (((c)>>24)&0xff)
#define COLOR_GET_RED(c)        (((c)>>16)&0xff)
#define COLOR_GET_GREEN(c)      (((c)>> 8)&0xff)
#define COLOR_GET_BLUE(c)       ( (c)     &0xff)

typedef enum tDisplayRequestType {
  displayrq_QUERY_AVAILABILITY,
  displayrq_QUERY_CURRENT_MODE,
  displayrq_SET_MODE
} tDisplayRequestType;

typedef enum tDisplayResponseType {
  displayrp_ERROR,
  displayrp_OK
} tDisplayResponseType;

typedef enum tDisplayColorDepth {
  dcd_4BPP = 4,
  dcd_8BPP = 8,
  dcd_15BPP = 15,
  dcd_16BPP = 16,
  dcd_24BPP = 24,
  dcd_32BPP = 32
} tDisplayColorDepth;

typedef struct tDisplay {
  uint Width, Height;
  tDisplayColorDepth ColorDepth;
  uint ModeNo;
  uint32 aOldVGAPalette[16];
#if DOS_DEBUG
  uint16 vsel;
#endif
} tDisplay;

typedef struct tDisplayInit {
  uint Width, Height;
  tDisplayColorDepth ColorDepth;
} tDisplayInit;

extern int  Display_Init (tDisplay* pDisplay, const tDisplayInit* pDisplayInit);
extern void Display_Done (tDisplay* pDisplay);

extern void Display_ThreadFunction (tDisplay* pDisplay);

extern void Display_Pixel   (tDisplay* pDisplay, int x, int y, uint32 color);
extern void Display_LineH   (tDisplay* pDisplay,
                             int x, int y, uint len, uint32 color);
extern void Display_LineV   (tDisplay* pDisplay,
                             int x, int y, uint len, uint32 color);
extern void Display_Line    (tDisplay* pDisplay,
                             int x1, int y1, int x2, int y2, uint32 color);
extern void Display_Box     (tDisplay* pDisplay, int x1, int y1, int x2, int y2,
                             int IsSolid, uint32 color);
extern void Display_Bitmap  (tDisplay* pDisplay, int x, int y,
                             const void* pBitmap, tDisplayColorDepth ColorDepth,
                             uint BitmapWidth, uint BitmapHeight,
                             uint x1, uint y1, uint x2, uint y2);
extern void Display_Text    (tDisplay* pDisplay, int x1, int y1, int x2, int y2,
                             int dx1, int dy1, int dx2, int dy2,
                             const char* pText, uint TextStrLen,
                             uint32 BackgroundColor, uint32 TextColor,
                             const uchar pFont8x16[256*16]);

#endif // _DISPLAY_H_
