#include "RectSet.h"

#define EXCHANGE(type,a,b)      {type tmp=a; a=b; b=tmp;}
#define SORT2(type,a1,a2)       {if (a1>a2) EXCHANGE(type,a1,a2)}
#if 0
#define SORT4(type,a1,a2,a3,a4) \
{                               \
  SORT2(type,a1,a2);            \
  SORT2(type,a3,a4);            \
  SORT2(type,a1,a3);            \
  SORT2(type,a2,a4);            \
  SORT2(type,a2,a3);            \
}
#else
#define SORT4(type,a1,a2,a3,a4) \
{                               \
  SORT2(type,a1,a2);            \
  SORT2(type,a3,a4);            \
  a3=MAX(a1,a3);                \
  a2=MIN(a2,a4);                \
  SORT2(type,a2,a3);            \
}
#endif

void RectSet_GetBoundRect (const tRectSetElement* pRect1, const tRectSetElement* pRect2, tRectSetElement* pBoundRect)
{
  pBoundRect->x1 = MIN (pRect1->x1, pRect2->x1);
  pBoundRect->y1 = MIN (pRect1->y1, pRect2->y1);
  pBoundRect->x2 = MAX (pRect1->x2, pRect2->x2);
  pBoundRect->y2 = MAX (pRect1->y2, pRect2->y2);
}

int  RectSet_GetIntersectRect (const tRectSetElement* pRect1, const tRectSetElement* pRect2, tRectSetElement* pIntersectRect)
{
  int x[4], y[4];

  if
  (
    (pRect1->x1 > pRect2->x2) ||
    (pRect2->x1 > pRect1->x2) ||
    (pRect1->y1 > pRect2->y2) ||
    (pRect2->y1 > pRect1->y2)
  )
  return 0;

  x[0] = pRect1->x1;
  x[1] = pRect1->x2;
  x[2] = pRect2->x1;
  x[3] = pRect2->x2;
  SORT4 (int, x[0], x[1], x[2], x[3]);
  pIntersectRect->x1 = x[1];
  pIntersectRect->x2 = x[2];

  y[0] = pRect1->y1;
  y[1] = pRect1->y2;
  y[2] = pRect2->y1;
  y[3] = pRect2->y2;
  SORT4 (int, y[0], y[1], y[2], y[3]);
  pIntersectRect->y1 = y[1];
  pIntersectRect->y2 = y[2];

  return 1;
}

int  RectSet_GetDifferenceRects (const tRectSetElement* pRect1, const tRectSetElement* pRect2, tRectSetElement pDiffRect[])
{
  tRectSetElement IntersectRect;
  enum {
    p_EMPTY  = 0,
    p_TOP    = 1,
    p_BOTTOM = 2,
    p_LEFT   = 4,
    p_RIGHT  = 8
  } p = p_EMPTY;

  if (!RectSet_GetIntersectRect (pRect1, pRect2, &IntersectRect))
  {
    pDiffRect[0] = *pRect1;
    return 1;
  }

  if (pRect1->x1 != IntersectRect.x1) p |= p_LEFT;
  if (pRect1->x2 != IntersectRect.x2) p |= p_RIGHT;
  if (pRect1->y1 != IntersectRect.y1) p |= p_TOP;
  if (pRect1->y2 != IntersectRect.y2) p |= p_BOTTOM;

  switch (p)
  {
    default:
    case p_EMPTY:
    return 0;

    case p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;
    return 1;

    case p_BOTTOM:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y1 = IntersectRect.y2+1;
    return 1;

    case p_BOTTOM|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].y1 = IntersectRect.y2+1;
    return 2;

    case p_LEFT:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].x2 = IntersectRect.x1-1;
    return 1;

    case p_LEFT|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].y1 = IntersectRect.y1;
      pDiffRect[1].x2 = IntersectRect.x1-1;
    return 2;

    case p_LEFT|p_BOTTOM:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].x2 = IntersectRect.x1-1;
      pDiffRect[0].y2 = IntersectRect.y2;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].y1 = IntersectRect.y2+1;
    return 2;

    case p_LEFT|p_BOTTOM|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = IntersectRect;
      pDiffRect[1].x1 = pRect1->x1;
      pDiffRect[1].x2 = IntersectRect.x1-1;

      pDiffRect[2] = *pRect1;
      pDiffRect[2].y1 = IntersectRect.y2+1;
    return 3;

    case p_RIGHT:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].x1 = IntersectRect.x2+1;
    return 1;

    case p_RIGHT|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].x1 = IntersectRect.x2+1;
      pDiffRect[1].y1 = IntersectRect.y1;
    return 2;

    case p_RIGHT|p_BOTTOM:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].x1 = IntersectRect.x2+1;
      pDiffRect[0].y2 = IntersectRect.y2;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].y1 = IntersectRect.y2+1;
    return 2;

    case p_RIGHT|p_BOTTOM|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = IntersectRect;
      pDiffRect[1].x1 = IntersectRect.x2+1;
      pDiffRect[1].x2 = pRect1->x2;

      pDiffRect[2] = *pRect1;
      pDiffRect[2].y1 = IntersectRect.y2+1;
    return 3;

    case p_RIGHT|p_LEFT:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].x2 = IntersectRect.x1-1;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].x1 = IntersectRect.x2+1;
    return 2;

    case p_RIGHT|p_LEFT|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].y1 = IntersectRect.y1;
      pDiffRect[1].x2 = IntersectRect.x1-1;

      pDiffRect[2] = *pRect1;
      pDiffRect[2].x1 = IntersectRect.x2+1;
      pDiffRect[2].y1 = IntersectRect.y1;
    return 3;

    case p_RIGHT|p_LEFT|p_BOTTOM:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].x2 = IntersectRect.x1-1;
      pDiffRect[0].y2 = IntersectRect.y2;

      pDiffRect[1] = *pRect1;
      pDiffRect[1].x1 = IntersectRect.x2+1;
      pDiffRect[1].y2 = IntersectRect.y2;

      pDiffRect[2] = *pRect1;
      pDiffRect[2].y1 = IntersectRect.y2+1;
    return 3;

    case p_RIGHT|p_LEFT|p_BOTTOM|p_TOP:
      pDiffRect[0] = *pRect1;
      pDiffRect[0].y2 = IntersectRect.y1-1;

      pDiffRect[1] = IntersectRect;
      pDiffRect[1].x1 = pRect1->x1;
      pDiffRect[1].x2 = IntersectRect.x1-1;

      pDiffRect[2] = IntersectRect;
      pDiffRect[2].x1 = IntersectRect.x2+1;
      pDiffRect[2].x2 = pRect1->x2;

      pDiffRect[3] = *pRect1;
      pDiffRect[3].y1 = IntersectRect.y2+1;
    return 4;
  }
}

#if 0
#include <stdio.h>
#include <conio.h>
void redraw (tRectSetElement* p1, tRectSetElement* p2, int IsIntersect)
{
  tRectSetElement a[4];
  int x, y, n, c;

  textbackground (BLACK);
  textcolor (LIGHTGRAY);
  clrscr();
  if (IsIntersect)
    cprintf ("Red 3s area is intersection of Blue 1s and Green 2s. Use <Space> to toggle.\r\n");
  else
    cprintf ("Red 3s area is Blue 1s minus Green 2s. Use <Space> to toggle.\r\n");
  cprintf ("Use arrow keys to move Green 2s around. <Esc> quits...\r\n");

  textbackground (BLACK);
  textcolor (BLUE);
  for(y=p1->y1; y<=p1->y2; y++)
    for(x=p1->x1; x<=p1->x2; x++)
    {
      gotoxy (x, y);
      cprintf ("1");
    }

  textbackground (BLACK);
  textcolor (GREEN);
  for(y=p2->y1; y<=p2->y2; y++)
    for(x=p2->x1; x<=p2->x2; x++)
    {
      gotoxy (x, y);
      cprintf ("2");
    }

  if (IsIntersect)
    c = RectSet_GetIntersectRect (p1, p2, a);
  else
    c = RectSet_GetDifferenceRects (p1, p2, a);

  textbackground (BLACK);
  textcolor (RED);
  for(n=0; n<c; n++)
  {
    for(y=a[n].y1; y<=a[n].y2; y++)
      for(x=a[n].x1; x<=a[n].x2; x++)
      {
        gotoxy (x, y);
        cprintf ("3");
      }
  }

  textbackground (BLACK);
  textcolor (LIGHTGRAY);
}

int main()
{
  tRectSetElement a={11,7,11+50,7+12};
  tRectSetElement b={65,12,65+5,12+4};
  int intersect=1, k;

  redraw (&a, &b, intersect);

  for(;;)
  {
    if (kbhit())
    {
      k = getch();
      if (!k)
        k = getch();
    }
    else
      k = 0;
    if (k==27) break;
    if (k)
    {
      switch (k)
      {
        case 'K':
          b.x1--;
          b.x2--;
        break;
        case 'M':
          b.x1++;
          b.x2++;
        break;
        case 'H':
          b.y1--;
          b.y2--;
        break;
        case 'P':
          b.y1++;
          b.y2++;
        break;
        case ' ':
          intersect ^= 1;
        break;
      }
      redraw (&a, &b, intersect);
    }
  }

  return 0;
}
#endif

