#ifndef _WIN_MNGR_H_
#define _WIN_MNGR_H_

#include "../ComTypes.h"
#include "../ComDefs.h"
#include "../Display/Display.h"
#include "RectSet.h"

typedef enum tWindowZGroup {
  wzg_DESKTOP_WALLPAPER,
  wzg_DESKTOP_ICON,
  wzg_WINDOW,
  wzg_WINDOW_ALWAYS_ON_TOP,
  wzg_DESKTOP_MENU_BAR,
  wzg_MOVE_RESIZE_FRAME,
  wzg_MOUSE_POINTER,

  wzg_COUNT = 1+wzg_MOUSE_POINTER
} tWindowZGroup;

typedef struct tWindowEvent {
  enum {
    wet_KEY,
    wet_MOUSE,
    wet_REDRAW,
    wet_CLOSE
  } EventType;
  union {
    uint16 KeyScanCode;
    struct {
      int X, Y, DeltaX, DeltaY, Buttons;
    } MouseState;
  } Data;
} tWindowEvent;

struct tWinManager;

typedef struct tWindow {
  tRectSetElement Rect;
  int IsMovable;
  int IsResizable;
  int IsHidden;
  tWindowZGroup ZGroup;
  struct tWindow* pUpper;
  struct tWindow* pLower;
  struct tWindow* pSubView;
  struct tWindow* pOwnerView;
  struct tWindow* pNextSubView;
  struct tWindow* pPrevSubView;
  void (*pDraw)(struct tWinManager*, struct tWindow*, const tRectSetElement*);
  void (*pEventHandler)(struct tWinManager*, struct tWindow*, const tWindowEvent*);
// private:
// ...
} tWindow;

typedef struct tMousePointerWindow {
  tWindow Window;
// private:
  const uint32* pMousePointerPixels;
} tMousePointerWindow;

#define WM_MAX_WINDOWS_FOR_MOUSE_POINTER        64

typedef struct tWinManager {
// Screen-related data
  tDisplay* pDisplay;
  uint ScrWidth, ScrHeight;
  tDisplayColorDepth ColorDepth;
  uint VirtScrWidth, VirtScrHeight;
  uint ScrPosX, ScrPosY;
// Bottom & top windows of Z-ordered groups of windows
  tWindow* aGroupTopWindows[wzg_COUNT];
  tWindow* aGroupBottomWindows[wzg_COUNT];
// Move/Resize frame windows (4 pcs)
  tWindow aMoveResizeWindows[4];
// Mouse-related data
  int MousePosX, MousePosY;
  tMousePointerWindow aMousePointerWindow[WM_MAX_WINDOWS_FOR_MOUSE_POINTER];
  uint MousePointerWindowCount;
} tWinManager;

typedef struct tWinManagerInit {
// Screen-related data
  tDisplay* pDisplay;
  uint ScrWidth, ScrHeight;
  tDisplayColorDepth ColorDepth;
  uint VirtScrWidth, VirtScrHeight;
  uint ScrPosX, ScrPosY;
} tWinManagerInit;

extern int  WinManager_Init (tWinManager* pWinManager, const tWinManagerInit* pWinManagerInit);
extern void WinManager_Done (tWinManager* pWinManager);
extern void WinManager_ThreadFunction (tWinManager* pWinManager);

extern void InitWindow (tWindow* pWindow, int x1, int y1, int x2, int y2,
                 tWindowZGroup ZGroup,
                 void (*pDraw)(struct tWinManager*, struct tWindow*, const tRectSetElement*),
                 void (*pEventHandler)(struct tWinManager*, struct tWindow*, const tWindowEvent*));
extern void InsertWindowSubView (tWindow* pWindow, tWindow* pSubView);
extern void DeleteWindowSubView (tWindow* pSubView);

extern void AddWindow (tWinManager* pWinManager, tWindow* pWindow);
extern void RemoveWindow (tWinManager* pWinManager, tWindow* pWindow);
extern void HideWindow (tWinManager* pWinManager, tWindow* pWindow);
extern void ShowWindow (tWinManager* pWinManager, tWindow* pWindow);
extern void SelectWindow (tWinManager* pWinManager, tWindow* pWindow);
extern void MoveWindow (tWinManager* pWinManager, tWindow* pWindow, int dx, int dy);
extern void DrawWindow (tWinManager* pWinManager, tWindow* pWindow);


#endif // _WIN_MNGR_H_
